import javax.swing.*;
import java.util.*;
import java.awt.*;
import java.awt.geom.*;
import java.io.PrintStream;

public class DigitalCircuit {
  private JPanel panel;
  private Simulator simulator;
  private ArrayList<Drawable> components;
  
  public DigitalCircuit(){
    simulator = new Simulator();
    components = new ArrayList<Drawable>();
  }
  
  public void setPanel(JPanel p){
    panel = p;
  }
  
  public void simulate(){
	// Start
	simulator.simulate();
	// ReSet
	simulator.reset();
	for (Drawable component : components) {
		if (component instanceof Gate) {
			((Gate) component).reset();
		}
		else if (component instanceof Source) {
			((Source) component).reset();
		}
		
	}
	
    
  }
  
  public Drawable find(Point p) {
     for (Drawable component : components)
       if (component.contains(p))
          return component;
     return null;
  }

  public GOutSignal findSource(Rectangle2D rec) {
     for (Drawable component : components) 
       if (component instanceof GOutSignal) {
          GOutSignal source = (GOutSignal)component;
          if (source.getGOutput().intersects(rec))
            return source;
       }
     return null;
  }
  
  public GInSignal findDrain(Rectangle2D rec) {
     for (Drawable component : components) 
       if (component instanceof GInSignal) {
          GInSignal drain = (GInSignal)component;
          if (drain.getGInput().intersects(rec))
            return drain;
       }
     return null;
  }

  public void paintComponent(Graphics2D g) {
    for (Drawable c: components) 
      c.draw(g);
  }
  
  public void addInverter() {
    components.add(new GInverter(new Point2D.Double(200f, 200f), simulator));
    panel.repaint();
  }
  
  public void addOrGate() {
    components.add(new GOrGate(new Point2D.Double(200f, 200f), simulator));
    panel.repaint();
  }
  public void addNOrGate() {
    components.add(new GNOrGate(new Point2D.Double(200f, 200f), simulator));
    panel.repaint();
  }
  public void addNAndGate() {
    components.add(new GNAndGate(new Point2D.Double(200f, 200f), simulator));
    panel.repaint();
  }
  public void addWire() {
    components.add(new GWire(new Point2D.Double(200f, 200f), this));
    panel.repaint();
  }
  
  public void addSource(Scanner data) {
    components.add(new GSource(new Point2D.Double(200f, 200f), data, simulator));
    panel.repaint();
  }
  
  public void addMeter(PrintStream os) {
    components.add(new GMeter(new Point2D.Double(200f, 200f), os,simulator));
    panel.repaint();
  }
  
}
