///////    Meter.java    ///////

/**
* Meter
* @author Felipe Acevedo - Cristobal Barrientos - Thomas Dixon
*/ 


import java.util.*;
import java.io.PrintStream;


public class Meter implements InSignal{
   /**
  * @param sim  Instancia de simulador
  * @param testPoints Puntos a medir 
  * @param out Salida
  * @param nexId siguiente Id del meter
  * @param id  Id del meter   
  */
	private Simulator sim;
	private ArrayList<OutSignal> testPoints;
	private GraphPanel out;
	private static int nextId=0;
   private final int id;
  
   {  id = nextId++;
   }
	
	/**
*Constructor  Meter 
* @param g		Instancia del GraphPanel
* @param s		Instancia del Simulador 
*/
	public Meter (GraphPanel g, Simulator s) {
		sim = s; 
		sim.addMeter(this);    
		out = g;
		testPoints = new ArrayList<OutSignal>();
   }
   
   /*public void addTestPoint (Wire w){
      testPoints.add(w);
   }*/
	
  /**
  * Chequea cambio de entrada
  */
	public void inputValueChanged()
	{
	return;
	}
	
  /**
  * Conecta  y agrega una entrada
  *@param output salida de una compuerta
  */	
	public void connectInput(OutSignal output){
	  testPoints.add(output);
	} 
		
  /**
  * Desconecta una entrada
  */
   public void disconnectInput()
	{
		while(testPoints.size() != 0)
		{
			testPoints.get(0).removeFromDrains(this);
			testPoints.remove(0); 
		}
	}
	
		
  /**
  * Agrega una entrada
  */
  
	public void addSource(OutSignal output)
	{
		testPoints.add(output);
	}
	
 /**
  *Remueve una seal de entrada
  *@param output Salida de una compuerta
  */
	public void removeFromSource(OutSignal output)
	{
		testPoints.remove(output);
	}
	
 /**
  *Verifica entrada conectada
  */ 	 
	public boolean hasConnectedSource() {
		return testPoints.size()>0;
	}
	
	  /**
  *Escribe el tiempo en el archivo de texto
  */	 
	public void writeHeader(){
		int i=0;
		for (OutSignal s:testPoints){ 
			out.setDescription(i, s.getDescription());
			if(i++ > 2)
				break;
		}
	}
	
	  /**
  *Escribe la salida
  */
	public void log(float time) {
		int i=0;
		if(time < out.getMaxPlotTime()){
			for (OutSignal s:testPoints){ 
				out.addPoint(i, time,(s.getValue()?1:0));
				if(i++ > 2)
					break;
			}
		}
   }
	  /**
  *Descripion de la Compuerta con el Id corresponiente
  */
	public String getDescription() {
    return getClass().getName()+"_"+id;
  }
}